package com.mif13.authServer.controllers;

import com.mif13.authServer.model.User;
import com.mif13.authServer.utils.JwtHelper;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.naming.AuthenticationException;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.util.MultiValueMap;
import org.springframework.util.MultiValueMapAdapter;
import org.springframework.web.bind.annotation.*;

@Controller
public class UserOperations {

    // TODO récupérer le DAO...

    /**
     * Procédure de login utilisée par un utilisateur
     *
     * @param login    Le login de l'utilisateur. L'utilisateur doit avoir été créé préalablement et
     *                 son login doit être présent dans le DAO.
     * @param password Le password à vérifier.
     * @return Une ResponseEntity avec le JWT dans le header "Authentication" si le login s'est bien
     * passé, et le code de statut approprié (204, 401 ou 404).
     */
    @PostMapping("/login")
    public ResponseEntity<Void> login(@RequestParam("login") String login,
        @RequestParam("password") String password, @RequestHeader("Origin") String origin) {
        String jwt = JwtHelper.generateToken(login, origin);

        ResponseEntity<Void> response;
        if (jwt != null && !jwt.isEmpty()) {
            // on ajoute un header Authentication avec comme valeur JWT
            Map<String, List<String>> headers = new HashMap<>();
            headers.put("Authentication", List.of(jwt));
            MultiValueMap<String, String> headersSpring = new MultiValueMapAdapter<>(headers);

            response = new ResponseEntity<>(headersSpring, HttpStatus.NO_CONTENT);
        } else {
            response = new ResponseEntity<>(HttpStatus.UNAUTHORIZED);
        }
        return response;
    }

    @GetMapping("/test")
    public User test() {
        return new User("mytestlogin", "mytestpassword");
    }

    @GetMapping("/test1")
    public ResponseEntity<User> test1() {
        String pwd = "TEST 1 Password";
        User user = new User("TEST 1 Login", pwd);
        try {
            user.authenticate(pwd);
        } catch (AuthenticationException e) {
            e.printStackTrace();
        }
        ResponseEntity<User> response = new ResponseEntity<User>(user, HttpStatus.OK);
        return response;
    }

    /**
     * Réalise la déconnexion
     */
    @PostMapping("/logout")
    public ResponseEntity<Void> logout() {
        // TODO
        ResponseEntity<Void> response = new ResponseEntity<Void>(HttpStatus.NO_CONTENT);
        return response;
    }

    /**
     * Méthode destinée au serveur Node pour valider l'authentification d'un utilisateur.
     *
     * @param jwt    Le token JWT qui se trouve dans le header "Authentication" de la requête
     * @param origin L'origine de la requête (pour la comparer avec celle du client, stockée dans le
     *               token JWT)
     * @return Une réponse vide avec un code de statut approprié (204, 400, 401).
     */
    @GetMapping("/authenticate")
    public ResponseEntity<Void> authenticate(@RequestParam("jwt") String jwt,
        @RequestParam("origin") String origin) {
        // TODO
        ResponseEntity<Void> response = new ResponseEntity<Void>(HttpStatus.NO_CONTENT);
        return response;
    }
}